@@ -16,6 +16,8 @@ from ob_dj_store.core.stores.models import (
     Store,
 )
 from ob_dj_store.core.stores.models._inventory import Inventory
+from ob_dj_store.core.stores.models._wallet import Wallet
+from ob_dj_store.core.stores.utils import validate_currency
 
 
 class StoreFilter(filters.FilterSet):
@@ -26,6 +28,7 @@ class StoreFilter(filters.FilterSet):
     search = filters.CharFilter(method="search_filter")
     shipping_methods = filters.Filter(method="by_shipping_methods")
     is_open = filters.BooleanFilter(method="by_open_stores")
+    country = filters.CharFilter(method="by_country")
 
     class Meta:
         model = Store
@@ -38,6 +41,9 @@ class StoreFilter(filters.FilterSet):
     def by_location(self, queryset, name, value):
         return queryset.filter(poly__contains=value)
 
+    def by_country(self, queryset, name, value):
+        return queryset.filter(address__country=value)
+
     def by_shipping_methods_names(self, queryset, name, value):
         return queryset.filter(
             shipping_methods__name__in=[
@@ -49,9 +55,15 @@ class StoreFilter(filters.FilterSet):
         if value:
             current_time = now()
             queryset = queryset.filter(
-                opening_hours__weekday=current_time.weekday() + 1,
-                opening_hours__from_hour__lte=current_time.time(),
-                opening_hours__to_hour__gte=current_time.time(),
+                Q(
+                    opening_hours__weekday=current_time.weekday() + 1,
+                    opening_hours__from_hour__lte=current_time.time(),
+                    opening_hours__to_hour__gte=current_time.time(),
+                )
+                | Q(
+                    opening_hours__weekday=current_time.weekday() + 1,
+                    opening_hours__always_open=True,
+                )
             )
         return queryset
 
@@ -112,28 +124,61 @@ class CategoryFilter(filters.FilterSet):
         ]
 
     def by_store(self, queryset, name, value):
-        return (
+        queryset = (
             queryset.filter(
-                subcategories__products__product_variants__inventories__store=value
+                Q(
+                    subcategories__products__product_variants__inventories__snooze_start_date__gt=now(),
+                    subcategories__products__product_variants__inventories__store=value,
+                    subcategories__products__is_active=True,
+                    menus__store=value,
+                    is_active=True,
+                )
+                | Q(
+                    subcategories__products__product_variants__inventories__snooze_end_date__lt=now(),
+                    subcategories__products__product_variants__inventories__store=value,
+                    menus__store=value,
+                    subcategories__products__is_active=True,
+                    is_active=True,
+                )
             )
             .prefetch_related(
                 Prefetch(
                     "subcategories",
                     queryset=Category.objects.filter(
-                        products__product_variants__inventories__store=value
+                        Q(
+                            products__product_variants__inventories__snooze_start_date__gt=now(),
+                            products__product_variants__inventories__store=value,
+                            products__is_active=True,
+                            is_active=True,
+                        )
+                        | Q(
+                            products__product_variants__inventories__snooze_end_date__lt=now(),
+                            products__product_variants__inventories__store=value,
+                            products__is_active=True,
+                            is_active=True,
+                        )
                     ).distinct(),
                 ),
                 Prefetch(
                     "subcategories__products",
                     queryset=Product.objects.filter(
-                        product_variants__inventories__store=value,
-                        is_active=True,
+                        Q(
+                            product_variants__inventories__snooze_start_date__gt=now(),
+                            product_variants__inventories__store=value,
+                            is_active=True,
+                        )
+                        | Q(
+                            product_variants__inventories__snooze_end_date__lt=now(),
+                            product_variants__inventories__store=value,
+                            is_active=True,
+                        )
                     ).distinct(),
                 ),
                 "subcategories__products__images",
             )
             .distinct()
         )
+        return queryset
 
     def by_type(self, queryset, name, value):
         return (
@@ -141,6 +186,12 @@ class CategoryFilter(filters.FilterSet):
                 subcategories__products__type=value,
             )
             .prefetch_related(
+                Prefetch(
+                    "subcategories",
+                    queryset=Category.objects.filter(
+                        is_active=True, products__type=value
+                    ).distinct(),
+                ),
                 Prefetch(
                     "subcategories__products",
                     queryset=Product.objects.filter(
@@ -205,10 +256,21 @@ class PaymentMethodFilter(filters.FilterSet):
     def by_digital(self, queryset, name, value):
         if value:
             return queryset.filter(
-                payment_provider__in=[
-                    store_settings.TAP_CREDIT_CARD,
-                    store_settings.TAP_KNET,
-                    store_settings.TAP_ALL,
-                ]
+                payment_provider__in=store_settings.DIGITAL_PAYMENT_METHODS
             )
         return queryset
+
+
+class WalletFilter(filters.FilterSet):
+    currency = filters.CharFilter(
+        method="by_currency",
+        validators=[
+            validate_currency,
+        ],
+    )
+
+    class Meta:
+        models = Wallet
+
+    def by_currency(self, queryset, name, value):
+        return queryset.filter(currency=value)
