@@ -1,16 +1,22 @@
 import calendar
+import json
+import logging
 import typing
-from datetime import time, timedelta
+from datetime import timedelta
+from decimal import Decimal
 
+import pycountry
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.gis.geos import Point
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
+from django.core.validators import validate_email
 from django.db import models
 from django.shortcuts import get_object_or_404
 from django.utils.module_loading import import_string
-from django.utils.timezone import now
+from django.utils.timezone import localtime, now
 from django.utils.translation import gettext_lazy as _
+from phonenumber_field.phonenumber import to_python
 from rest_framework import serializers
 
 from config import settings as store_settings
@@ -47,6 +53,8 @@ from ob_dj_store.core.stores.models import (
 from ob_dj_store.core.stores.models._inventory import Inventory
 from ob_dj_store.core.stores.utils import distance
 
+logger = logging.getLogger(__name__)
+
 
 class AttributeChoiceSerializer(serializers.ModelSerializer):
     class Meta:
@@ -220,89 +228,141 @@ class OrderSerializer(serializers.ModelSerializer):
             raise serializers.ValidationError(_("Store does not exist!"))
         return store
 
+    def _validate_user_address(
+        self,
+        attrs,
+    ):
+        if "shipping_address" not in attrs:
+            raise ValidationError(
+                {"shipping_address": _("Please provide a delivery address.")}
+            )
+        shipping_address = attrs["shipping_address"]
+        store = attrs["store"]
+        if store.poly:
+            if not store.poly.contains(shipping_address.location):
+                raise ValidationError(
+                    {
+                        "shipping_address": _(
+                            "The shipping address you provided is outside of the store's delivery range."
+                        )
+                    }
+                )
+        else:
+            logger.error(f"Store {store.name} support delivery but doesn't have a poly")
+            raise ValidationError(
+                {"store": _("Delivery is not supported by this store.")}
+            )
+
+    def _validate_digital_product(self, gift_details, attrs):
+        errors = []
+        for key in settings.DIGITAL_PRODUCTS_REQUIRED_KEYS:
+            if not (key in gift_details.keys() and len(str(gift_details.get(key)))):
+                errors.append({key: "This field should be filled."})
+        if len(errors) > 0:
+            raise serializers.ValidationError(errors)
+        email = gift_details.get("email")
+        phone_number = gift_details.get("phone_number")
+
+        if email and phone_number:
+            raise serializers.ValidationError(
+                _("Both Email and Phone number cannot be provided.")
+            )
+        elif not email and not phone_number:
+            raise serializers.ValidationError(_("Email or Phone number is required."))
+        elif phone_number and not to_python(phone_number).is_valid():
+            raise serializers.ValidationError(_("Invalid Phone number format."))
+        elif email:
+            try:
+                validate_email(email)
+            except ValidationError:
+                raise serializers.ValidationError(_("Invalid Email format."))
+
+        if not pycountry.currencies.get(alpha_3=gift_details["currency"]):
+            raise serializers.ValidationError("Gift currency is not valid")
+        try:
+            product = Product.objects.get(id=gift_details["digital_product"])
+        except ObjectDoesNotExist:
+            raise serializers.ValidationError(_("Digital product does not exist!"))
+        if product.type == Product.ProductTypes.PHYSICAL:
+            raise serializers.ValidationError(
+                _("You must not fill extra infos for physical products.")
+            )
+
+        gift_minimun_amount = store_settings.GIFT_MINIMUM_AMOUNT.get(
+            attrs["store"].currency, 5
+        )
+        if gift_details["price"] <= gift_minimun_amount - 1:
+            raise serializers.ValidationError(_("Gift card amount is not valid!"))
+
+    def _validate_pickup_time(self, store, pickup_time):
+        # validate that the pickup_time is always in the future
+        if pickup_time < localtime(now()):
+            raise serializers.ValidationError(_("Pickup time must be in the future"))
+        pickup_time_time = pickup_time.time()
+        # validate that the pickup_time is between store's opening hours and closing hours
+        try:
+            op_hour = store.opening_hours.get(weekday=pickup_time.weekday() + 1)
+            if (
+                not op_hour.always_open
+                and pickup_time_time > op_hour.to_hour
+                or pickup_time_time < op_hour.from_hour
+            ):
+                raise serializers.ValidationError(
+                    _("Pickup time must be between store's opening hours")
+                )
+        except ObjectDoesNotExist:
+            logger.error(f"{store.name} Store doesn't have opening hours for ")
+
     def validate(self, attrs):
         user = self.context["request"].user
         attrs["store"] = self._get_store()
         if not "extra_infos" in attrs:
             attrs["extra_infos"] = {}
         stores = Store.objects.filter(store_items__cart=user.cart).distinct()
-        if not store_settings.DIFFERENT_STORE_ORDERING and len(stores) > 1:
-            raise ValidationError(_("You cannot order from different stores"))
         gift_details = attrs["extra_infos"].get("gift_details", None)
+        apple_pay_attr = attrs["extra_infos"].get("apple_pay", None)
+        if apple_pay_attr:
+            token_data = apple_pay_attr.encode("utf-8").decode("unicode_escape")
+            new_token_data = json.loads(token_data)
+            attrs["extra_infos"]["apple_pay"] = new_token_data
         if gift_details:
-            errors = []
-            for key in settings.DIGITAL_PRODUCTS_REQUIRED_KEYS:
-                if not (key in gift_details.keys() and len(str(gift_details.get(key)))):
-                    errors.append({key: "This field should be filled."})
-            if len(errors) > 0:
-                raise serializers.ValidationError(errors)
-            try:
-                product = Product.objects.get(id=gift_details["digital_product"])
-            except ObjectDoesNotExist:
-                raise serializers.ValidationError(_("Digital product does not exist!"))
-            if product.type == Product.ProductTypes.PHYSICAL:
-                raise serializers.ValidationError(
-                    _("You must not fill extra infos for physical products.")
-                )
+            self._validate_digital_product(gift_details, attrs)
+            currency = gift_details["currency"]
         # The Cart items must not be empty
         elif not user.cart.items.exists():
             raise serializers.ValidationError(_("The Cart must not be empty"))
+        else:
+            if not store_settings.DIFFERENT_STORE_ORDERING:
+                if len(stores) > 1:
+                    raise ValidationError(_("You cannot order from different stores"))
+                for item in user.cart.items.all():
+                    if not item.inventory:
+                        product_name = item.product_variant.product.name
+                        variant_name = item.product_variant.name
+                        message = (
+                            f"{product_name} {variant_name} doesn't exist on this store"
+                        )
+                        raise ValidationError(_(message))
 
-        if "pickup_time" in attrs:
-            if attrs["pickup_time"]:
-                # validate that the pickup_time is always in the future
-                if attrs["pickup_time"] < now():
-                    raise serializers.ValidationError(
-                        _("Pickup time must be in the future")
-                    )
-                # validate that the pickup_time is part of day (between 00:00 and 23:59)
-                if not (
-                    attrs["pickup_time"].time() >= time(hour=0, minute=0)
-                    and attrs["pickup_time"].time() <= time(hour=23, minute=59)
-                ):
-                    raise serializers.ValidationError(
-                        _("Pickup time must be part of day")
-                    )
-                # validate that the pickup_time is between store's opening hours and closing hours
+            if "shipping_method" in attrs:
                 if (
-                    attrs["store"]
-                    .opening_hours.filter(weekday=attrs["pickup_time"].weekday() + 1)
-                    .exists()
-                ) and (
-                    attrs["pickup_time"].hour
-                    > (
-                        attrs["store"]
-                        .opening_hours.filter(
-                            weekday=attrs["pickup_time"].weekday() + 1
-                        )
-                        .first()
-                        .to_hour.hour
-                    )
-                    or attrs["pickup_time"].hour
-                    < (
-                        attrs["store"]
-                        .opening_hours.filter(
-                            weekday=attrs["pickup_time"].weekday() + 1
-                        )
-                        .first()
-                        .from_hour.hour
-                    )
+                    attrs["shipping_method"].type
+                    == ShippingMethod.ShippingType.DELIVERY
                 ):
-                    raise serializers.ValidationError(
-                        _("Pickup time must be between store's opening hours")
-                    )
+                    self._validate_user_address(attrs)
+            currency = attrs["store"].currency
+
+        if attrs.get("pickup_time", None):
+            self._validate_pickup_time(attrs["store"], attrs["pickup_time"])
+
         payment_method = attrs.get("payment_method")
         if payment_method:
             if payment_method.payment_provider == store_settings.WALLET:
                 try:
-                    wallet = user.wallets.get(currency=attrs["store"].currency)
+                    wallet = user.wallets.get(currency=currency)
                 except ObjectDoesNotExist:
                     raise serializers.ValidationError(
-                        {
-                            "wallet": _(
-                                "The currency of our store is not compatible with your wallet. Please checkout with products using different currency"
-                            )
-                        }
+                        {"wallet": _(f"{currency} is not a valid currency")}
                     )
 
                 if gift_details:
@@ -321,7 +381,7 @@ class OrderSerializer(serializers.ModelSerializer):
 
         return super().validate(attrs)
 
-    def perform_payment(self, amount, payment_method, order_store, orders):
+    def perform_payment(self, amount, payment_method, order_store, orders, currency):
         from ob_dj_store.core.stores.gateway.tap.utils import TapException
 
         try:
@@ -329,16 +389,21 @@ class OrderSerializer(serializers.ModelSerializer):
                 user=self.context["request"].user,
                 amount=amount,
                 method=payment_method,
-                currency=order_store.currency,
+                currency=currency,
                 orders=orders,
             )
+            tap_transaction = payment.tap_payment
         except ValidationError as err:
-            raise serializers.ValidationError(detail=err.message_dict)
+            raise serializers.ValidationError(detail=err.messages)
         except TapException as err:
             raise serializers.ValidationError({"tap": _(str(err))})
+        except ObjectDoesNotExist:
+            tap_transaction = None
+
         return {
             "orders": orders,
             "payment_url": payment.payment_url,
+            "charge_id": tap_transaction.charge_id if tap_transaction else None,
         }
 
     def create(self, validated_data: typing.Dict):
@@ -350,9 +415,9 @@ class OrderSerializer(serializers.ModelSerializer):
             amount = gift_details["price"]
             order = Order.objects.create(customer=user, **validated_data)
             orders.append(order)
+            currency = gift_details["currency"]
         else:
             cart = user.cart
-            amount = cart.total_price
             stores = Store.objects.filter(store_items__cart=cart).distinct()
             orders = []
             validated_data.pop("store")
@@ -366,21 +431,29 @@ class OrderSerializer(serializers.ModelSerializer):
                         order=order,
                         product_variant=item.product_variant,
                         quantity=item.quantity,
+                        attributes=list(item.attribute_choices.all()),
+                        notes=item.notes,
                     )
-                    order_item.attribute_choices.set(item.attribute_choices.all())
                 orders.append(order)
+            amount = Decimal(
+                sum(map(lambda order: Decimal(order.total_amount) or 0, orders))
+            )
+            currency = order_store.currency
+
         payment_method = validated_data.get("payment_method")
         return self.perform_payment(
             amount=amount,
             payment_method=payment_method,
             order_store=order_store,
             orders=orders,
+            currency=currency,
         )
 
 
 class CreateOrderResponseSerializer(serializers.Serializer):
     orders = OrderSerializer(many=True, read_only=True)
     payment_url = serializers.CharField(read_only=True)
+    charge_id = serializers.CharField(read_only=True)
     extra_infos = serializers.JSONField(
         required=False,
         help_text=f"""
@@ -392,7 +465,7 @@ class CreateOrderResponseSerializer(serializers.Serializer):
     car_id = serializers.IntegerField(required=False)
 
     # write only fields
-    shipping_method = serializers.IntegerField(write_only=True, required=False)
+    shipping_method = serializers.IntegerField(write_only=True, required=True)
     payment_method = serializers.IntegerField(write_only=True, required=False)
     shipping_address = serializers.IntegerField(write_only=True, required=False)
     pickup_time = serializers.DateTimeField(write_only=True, required=False)
@@ -436,12 +509,15 @@ class ProductVariantSerializer(serializers.ModelSerializer):
         fields = (
             "id",
             "name",
+            "label",
             "sku",
             "product_attributes",
             "is_primary",
             "inventory",
             "image",
             "image_thumbnail_medium",
+            "description",
+            "is_special",
         )
         extra_kwargs = {"image_thumbnail_medium": {"read_only": True}}
 
@@ -465,6 +541,9 @@ class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
     is_uncountable = serializers.SerializerMethodField()
     is_available_in_store = serializers.SerializerMethodField()
     product_id = serializers.SerializerMethodField()
+    favorite = serializers.SerializerMethodField()
+    is_favorite = serializers.SerializerMethodField()
+    is_multi_variant = serializers.SerializerMethodField()
 
     class Meta:
         model = CartItem
@@ -484,6 +563,9 @@ class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
             "inventory_quantity",
             "is_uncountable",
             "is_available_in_store",
+            "favorite",
+            "is_favorite",
+            "is_multi_variant",
         )
         extra_kwargs = {
             "store": {
@@ -491,6 +573,10 @@ class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
             },
         }
 
+    def get_is_multi_variant(self, obj):
+        product = obj.product_variant.product
+        return product.product_variants.all().count() > 1
+
     def get_is_available_in_store(self, obj):
         return True if obj.inventory else False
 
@@ -517,7 +603,30 @@ class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
         else:
             return None
 
+    def _get_favorite(self, obj):
+        user = self.context["request"].user
+        favorites = Favorite.objects.favorites_for_object(
+            obj.product_variant.product, user
+        )
+        customization = [
+            obj.product_variant,
+        ] + [attribute_choice for attribute_choice in obj.attribute_choices.all()]
+        for favorite in favorites:
+            content_objects = [
+                instance.content_object for instance in favorite.extras.all()
+            ]
+            if set(customization) == set(content_objects):
+                return favorite.id
+        return None
+
+    def get_favorite(self, obj) -> int:
+        return self.favorite
+
+    def get_is_favorite(self, obj) -> bool:
+        return self.favorite is not None
+
     def to_representation(self, instance: CartItem):
+        self.favorite = self._get_favorite(instance)
         data = super().to_representation(instance)
         data["product_variant"] = ProductVariantSerializer(
             instance=instance.product_variant
@@ -627,6 +736,7 @@ class ProductSerializer(FavoriteMixin, serializers.ModelSerializer):
             "id",
             "name",
             "slug",
+            "label",
             "description",
             "product_images",
             "product_variants",
@@ -649,6 +759,7 @@ class ProductListSerializer(serializers.ModelSerializer):
             "id",
             "name",
             "slug",
+            "label",
             "description",
             "product_images",
             "type",
@@ -659,6 +770,7 @@ class SubCategorySerializer(serializers.ModelSerializer):
     products = ProductListSerializer(many=True)
     image_thumbnail_medium = serializers.ImageField(read_only=True)
     image_thumbnail_small = serializers.ImageField(read_only=True)
+    is_available = serializers.SerializerMethodField()
 
     class Meta:
         model = Category
@@ -672,16 +784,30 @@ class SubCategorySerializer(serializers.ModelSerializer):
             "image_thumbnail_medium",
             "image_thumbnail_small",
             "parent",
+            "is_available",
         )
 
     def to_representation(self, instance):
         data = super().to_representation(instance)
         return data
 
+    def get_is_available(self, obj) -> bool:
+        store_id = self.context["request"].query_params.get("store", None)
+        if store_id:
+            for availability_hours in obj.parent.availability_hours.all():
+                if availability_hours.category == obj.parent:
+                    return (
+                        availability_hours.from_hour
+                        <= localtime(now()).time()
+                        <= availability_hours.to_hour
+                    )
+        return False
+
 
 class CategorySerializer(serializers.ModelSerializer):
     products = ProductListSerializer(many=True)
     subcategories = SubCategorySerializer(many=True, read_only=True)
+    is_available = serializers.SerializerMethodField()
 
     class Meta:
         model = Category
@@ -696,8 +822,21 @@ class CategorySerializer(serializers.ModelSerializer):
             "image",
             "image_thumbnail_medium",
             "image_thumbnail_small",
+            "is_available",
         )
 
+    def get_is_available(self, obj) -> bool:
+        store_id = self.context["request"].query_params.get("store", None)
+        if store_id:
+            for availability_hours in obj.availability_hours.all():
+                if availability_hours.category == obj:
+                    return (
+                        availability_hours.from_hour
+                        <= localtime(now()).time()
+                        <= availability_hours.to_hour
+                    )
+        return False
+
 
 class FeedbackConfigSerializer(serializers.ModelSerializer):
     category = CategorySerializer(many=False)
@@ -827,15 +966,20 @@ class StoreSerializer(FavoriteMixin, serializers.ModelSerializer):
             "phone_contacts",
             "current_day_opening_hours",
             "image",
+            "busy_mode",
         )
         extra_kwargs = {
             "image": {"read_only": True, "required": False},
         }
 
     def get_is_closed(self, obj):
-        current_time = now()
+        if obj.busy_mode:
+            return True
+        current_time = localtime(now())
         current_op_hour = obj.current_opening_hours
         if current_op_hour:
+            if current_op_hour.always_open:
+                return False
             return (
                 not current_op_hour.from_hour
                 <= current_time.time()
@@ -858,9 +1002,23 @@ class StoreSerializer(FavoriteMixin, serializers.ModelSerializer):
 
     def get_opening_hours(self, obj):
         opening_hours = sorted(
-            list(obj.opening_hours.all()), key=lambda op_hour: op_hour.weekday
+            [op_hour for op_hour in obj.opening_hours.all() if not op_hour.always_open],
+            key=lambda op_hour: op_hour.weekday,
+        )
+        always_open_op_hours = sorted(
+            [op_hour for op_hour in obj.opening_hours.all() if op_hour.always_open],
+            key=lambda op_hour: op_hour.weekday,
         )
         formatted_hours = []
+        if len(always_open_op_hours) > 0:
+            if len(always_open_op_hours) > 1:
+                formatted_hours.append(
+                    f"{calendar.day_name[always_open_op_hours[0].weekday - 1]} - {calendar.day_name[always_open_op_hours[-1].weekday - 1]} 24 hours"
+                )
+            else:
+                formatted_hours.append(
+                    f"{calendar.day_name[always_open_op_hours[0].weekday - 1]} 24 hours"
+                )
         if len(opening_hours) > 0:
             day_1 = calendar.day_name[opening_hours[0].weekday - 1]
             day_2 = None
@@ -907,13 +1065,15 @@ class StoreSerializer(FavoriteMixin, serializers.ModelSerializer):
 
     def get_current_day_opening_hours(self, obj):
         current_opening_hours = obj.current_opening_hours
-        op_hour = {}
         if current_opening_hours:
-            op_hour["from_hour"] = current_opening_hours.from_hour.strftime("%I:%M%p")
-            op_hour["to_hour"] = current_opening_hours.to_hour.strftime("%I:%M%p")
+            if current_opening_hours.always_open:
+                op_hour = "24 hours"
+            else:
+                from_hour = current_opening_hours.from_hour.strftime("%I:%M%p")
+                to_hour = current_opening_hours.to_hour.strftime("%I:%M%p")
+                op_hour = f"{from_hour} - {to_hour}"
         else:
-            op_hour["from_hour"] = settings.DEFAULT_OPENING_HOURS[0]["from_hour"]
-            op_hour["to_hour"] = settings.DEFAULT_OPENING_HOURS[0]["to_hour"]
+            op_hour = f"{settings.DEFAULT_OPENING_HOURS[0]['from_hour']} - {settings.DEFAULT_OPENING_HOURS[0]['to_hour']}"
         return op_hour
 
     def to_representation(self, instance):
@@ -1185,31 +1345,48 @@ class WalletMediaSerializer(serializers.ModelSerializer):
 class WalletSerializer(serializers.ModelSerializer):
     class Meta:
         model = Wallet
-        fields = ["id", "user", "balance", "name", "media_image", "image_url"]
+        fields = [
+            "id",
+            "user",
+            "currency",
+            "balance",
+            "name",
+            "media_image",
+            "image_url",
+        ]
         extra_kwargs = {
             "user": {"read_only": True},
+            "currency": {"read_only": True},
         }
 
 
 class WalletTopUpSerializer(serializers.Serializer):
     amount = serializers.DecimalField(
-        max_digits=10, decimal_places=2, min_value=1, required=True
+        max_digits=10, decimal_places=2, min_value=1, required=True, write_only=True
     )
     payment_method = serializers.PrimaryKeyRelatedField(
+        write_only=True,
         queryset=PaymentMethod.objects.filter(
             payment_provider__in=[
                 store_settings.TAP_CREDIT_CARD,
                 store_settings.TAP_KNET,
                 store_settings.TAP_ALL,
+                store_settings.APPLE_PAY,
             ]
         ),
         required=True,
     )
+    extra_infos = serializers.JSONField(required=False, write_only=True)
+    payment_url = serializers.CharField(read_only=True)
+    charge_id = serializers.CharField(read_only=True)
 
     def top_up_wallet(self, wallet):
         amount = self.validated_data["amount"]
         payment_method = self.validated_data["payment_method"]
-        return wallet.top_up_wallet(amount=amount, payment_method=payment_method)
+        extra_infos = self.validated_data.get("extra_infos", {})
+        return wallet.top_up_wallet(
+            amount=amount, payment_method=payment_method, extra_infos=extra_infos
+        )
 
 
 class WalletTransactionSerializer(serializers.ModelSerializer):
@@ -1221,6 +1398,8 @@ class WalletTransactionSerializer(serializers.ModelSerializer):
             "amount",
             "created_at",
             "wallet",
+            "is_cashback",
+            "is_refund",
         ]
 
 
