@@ -3,10 +3,11 @@ import typing
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
-from django.db.models import Max, Prefetch
+from django.db.models import Max, Prefetch, Q
 from django.http import Http404
 from django.shortcuts import get_object_or_404
 from django.utils.decorators import method_decorator
+from django.utils.timezone import localtime, now
 from django.utils.translation import ugettext_lazy as _
 from django_auto_prefetching import AutoPrefetchViewSetMixin, prefetch
 from django_filters.rest_framework import DjangoFilterBackend
@@ -29,6 +30,7 @@ from ob_dj_store.apis.stores.filters import (
     ProductFilter,
     StoreFilter,
     VariantFilter,
+    WalletFilter,
 )
 from ob_dj_store.apis.stores.rest.serializers.serializers import (
     CartItemSerializer,
@@ -55,6 +57,8 @@ from ob_dj_store.apis.stores.rest.serializers.serializers import (
 )
 from ob_dj_store.core.stores.gateway.tap.utils import TapException
 from ob_dj_store.core.stores.models import (
+    AttributeChoice,
+    AvailabilityHours,
     Cart,
     CartItem,
     Category,
@@ -277,7 +281,7 @@ class StoreView(
             .distinct()
             .count()
         )
-        nearby_count = super().get_queryset().count()
+        nearby_count = self.filter_queryset(self.get_queryset()).count()
         favorite_store_ids = Favorite.objects.favorites_for_model(
             Store, self.request.user
         ).values_list("object_id", flat=True)
@@ -343,6 +347,43 @@ class CartView(
     def update(self, request, *args, **kwargs):
         return super().update(request, *args, **kwargs)
 
+    @swagger_auto_schema(
+        operation_summary="Validate Customer Cart",
+        operation_description="""
+            Validate the current customer's cart
+        """,
+        manual_parameters=[
+            openapi.Parameter(
+                "store",
+                in_=openapi.IN_QUERY,
+                description="Store of the order",
+                type=openapi.TYPE_INTEGER,
+                required=not store_settings.DIFFERENT_STORE_ORDERING,
+            ),
+        ],
+        tags=[
+            "Cart",
+        ],
+    )
+    @action(
+        methods=["POST"],
+        detail=False,
+        url_path=r"validate",
+        permission_classes=[
+            permissions.IsAuthenticated,
+        ],
+        serializer_class=OrderSerializer,
+    )
+    def validate(self, request, *args, **kwargs):
+        store_pk = request.data.get("store", None)
+        if not store_settings.DIFFERENT_STORE_ORDERING and not store_pk:
+            raise ValidationError({"store": "Store field is missing!"})
+        if store_pk:
+            self.kwargs["store_pk"] = store_pk
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        return Response({"result": "ok"}, status=status.HTTP_200_OK)
+
 
 class CartItemView(
     mixins.RetrieveModelMixin,
@@ -457,13 +498,14 @@ class OrderView(
         serializer = OrderSerializer(data=request.data, context=context)
         orders = []
         if serializer.is_valid(raise_exception=True):
-            orders, payment_url = serializer.save().values()
+            orders, payment_url, charge_id = serializer.save().values()
         orders_data = OrderSerializer(orders, many=True).data
         return Response(
             status=status.HTTP_201_CREATED,
             data={
                 "payment_url": payment_url,
                 "orders": orders_data,
+                "charge_id": charge_id,
             },
         )
 
@@ -579,17 +621,45 @@ class ProductView(
     queryset = Product.objects.active()
 
     def apply_prefetch(self, queryset):
-        return queryset.prefetch_related(
+
+        qs = queryset.prefetch_related(
             Prefetch(
                 "product_variants",
                 queryset=ProductVariant.objects.filter(
-                    inventories__store__pk=self.kwargs["store_pk"]
+                    Q(
+                        inventories__snooze_start_date__gt=now(),
+                        inventories__store__pk=self.kwargs["store_pk"],
+                    )
+                    | Q(
+                        inventories__snooze_end_date__lt=now(),
+                        inventories__store__pk=self.kwargs["store_pk"],
+                    )
+                ).distinct(),
+            ),
+            Prefetch(
+                "product_variants__product_attributes__attribute_choices",
+                queryset=AttributeChoice.objects.filter(
+                    Q(
+                        attribute_inventory__snooze_start_date__gt=now(),
+                        attribute_inventory__store__pk=self.kwargs["store_pk"],
+                    )
+                    | Q(
+                        attribute_inventory__snooze_end_date__lt=now(),
+                        attribute_inventory__store__pk=self.kwargs["store_pk"],
+                    )
                 ).distinct(),
             ),
         ).distinct()
+        return qs
 
     def get_queryset(self):
-        queryset = super().get_queryset()
+        queryset = (
+            super()
+            .get_queryset()
+            .prefetch_related(
+                "images",
+            )
+        )
         if self.action == "favorites":
             favorite_product_ids = Favorite.objects.favorites_for_model(
                 Product, self.request.user
@@ -700,12 +770,25 @@ class CategoryViewSet(
     http_method_names = ["get"]
     serializer_class = CategorySerializer
     permission_classes = (permissions.AllowAny,)
-    queryset = Category.objects.active()
+    queryset = Category.objects.all()
     filterset_class = CategoryFilter
     lookup_value_regex = "[0-9]+"
 
     def get_queryset(self):
-        return super().get_queryset().prefetch_related("products__images")
+        store_id = self.request.query_params.get("store", None)
+        queryset = super().get_queryset()
+        if store_id:
+            queryset = queryset.prefetch_related(
+                "products__images",
+                Prefetch(
+                    "availability_hours",
+                    queryset=AvailabilityHours.objects.filter(
+                        weekday=localtime(now()).weekday() + 1,
+                        store=store_id,
+                    ),
+                ),
+            )
+        return queryset.filter(is_active=True)
 
     def get_object(self):
         store_id = self.request.query_params.get("store", None)
@@ -713,15 +796,32 @@ class CategoryViewSet(
         if not store_id:
             return get_object_or_404(Category, pk=instance_pk, is_active=True)
         try:
-            product_queryset = Product.objects.filter(
+            not_in_range = Q(
+                product_variants__inventories__snooze_start_date__gt=now(),
+                product_variants__inventories__store=store_id,
+                is_active=True,
+            ) | Q(
+                product_variants__inventories__snooze_end_date__lt=now(),
                 product_variants__inventories__store=store_id,
                 is_active=True,
             )
+            product_queryset = Product.objects.filter(not_in_range).distinct()
             instance = self.queryset.prefetch_related(
                 Prefetch(
                     "subcategories",
                     queryset=Category.objects.filter(
-                        products__product_variants__inventories__store=store_id
+                        Q(
+                            products__product_variants__inventories__snooze_start_date__gt=now(),
+                            products__product_variants__inventories__store=store_id,
+                            products__is_active=True,
+                            is_active=True,
+                        )
+                        | Q(
+                            products__product_variants__inventories__snooze_end_date__lt=now(),
+                            products__product_variants__inventories__store=store_id,
+                            products__is_active=True,
+                            is_active=True,
+                        )
                     ).distinct(),
                 ),
                 Prefetch(
@@ -1021,14 +1121,15 @@ class WalletViewSet(
 ):
     queryset = Wallet.objects.all()
     serializer_class = WalletSerializer
+    filterset_class = WalletFilter
     permission_classes = [
         permissions.IsAuthenticated,
     ]
 
     def get_queryset(self):
-        return Wallet.objects.filter(user=self.request.user).select_related(
-            "media_image"
-        )
+        return Wallet.objects.filter(
+            user=self.request.user, is_active=True
+        ).select_related("media_image")
 
     @swagger_auto_schema(
         operation_summary="Get User Wallet",
@@ -1093,10 +1194,13 @@ class WalletViewSet(
         serializer.is_valid(raise_exception=True)
         instance = self.get_object()
         try:
-            payment_url = serializer.top_up_wallet(instance)
+            payment_url, charge_id = serializer.top_up_wallet(instance)
         except TapException as err:
             raise ValidationError({"tap": _(f"{str(err)}")})
-        return Response({"payment_url": payment_url}, status=status.HTTP_200_OK)
+        return Response(
+            {"payment_url": payment_url, "charge_id": charge_id},
+            status=status.HTTP_200_OK,
+        )
 
     @swagger_auto_schema(
         operation_summary="List Wallet's transactions",
