@@ -1,15 +1,23 @@
+import typing
+
 from django import forms
 from django.contrib import admin
+from django.utils.translation import gettext_lazy as _
+from import_export.admin import ImportExportModelAdmin
 from leaflet.admin import LeafletGeoAdmin
 
 from ob_dj_store.core.stores import models
 from ob_dj_store.core.stores.admin_inlines import (
+    AttributeChoiceInlineAdmin,
+    AvailabilityHoursInlineAdmin,
     CartItemInlineAdmin,
     InventoryInlineAdmin,
     InventoryOperationInlineAdmin,
     OpeningHoursInlineAdmin,
+    OrderHistoryInlineAdmin,
     OrderItemInline,
     PhoneContactInlineAdmin,
+    ProductAttributeInlineAdmin,
     ProductMediaInlineAdmin,
     ProductVariantInlineAdmin,
 )
@@ -51,6 +59,7 @@ class StoreAdmin(LeafletGeoAdmin):
         "name",
         "location",
         "is_active",
+        "busy_mode",
         "currency",
         "minimum_order_amount",
         "delivery_charges",
@@ -79,6 +88,7 @@ class StoreAdmin(LeafletGeoAdmin):
                     "payment_methods",
                     "pickup_addresses",
                     "image",
+                    "currency",
                 )
             },
         ),
@@ -96,6 +106,9 @@ class StoreAdmin(LeafletGeoAdmin):
 
 
 class CategoryAdmin(admin.ModelAdmin):
+    inlines = [
+        AvailabilityHoursInlineAdmin,
+    ]
     list_display = ["name", "is_active", "parent", "image"]
     search_fields = [
         "name",
@@ -104,28 +117,68 @@ class CategoryAdmin(admin.ModelAdmin):
         "is_active",
     ]
 
+    def get_queryset(self, request):
+        return (
+            super()
+            .get_queryset(request)
+            .prefetch_related(
+                "availability_hours",
+            )
+        )
+
 
 class ProductVariantAdmin(admin.ModelAdmin):
     inlines = [
         InventoryInlineAdmin,
+        ProductAttributeInlineAdmin,
     ]
     list_display = [
         "name",
+        "label",
         "product",
+        "is_special",
         "has_inventory",
     ]
     search_fields = ["name", "product__name", "sku"]
+    exclude = ("product_attributes",)
 
     def get_queryset(self, request):
-        return super().get_queryset(request).prefetch_related("inventories")
+        return (
+            super()
+            .get_queryset(request)
+            .select_related("product")
+            .prefetch_related(
+                "inventories",
+                "inventories__store",
+                "product_attributes",
+                "product_attributes__attribute_choices",
+            )
+        )
+
+
+class ProductAdminForm(forms.ModelForm):
+    class Meta:
+        model = models.Product
+        fields = "__all__"
+
+    def clean_category(self):
+        categories = self.cleaned_data.get("category")
+        for category in categories:
+            if not category.parent:
+                raise forms.ValidationError(_("Category Doesn't have a parent"))
+        return categories
+
+
+class MenuAdmin(admin.ModelAdmin):
+    list_display = ["id", "store", "name", "created_at"]
+    search_fields = [
+        "store",
+    ]
 
 
 class ProductAdmin(admin.ModelAdmin):
-    list_display = ["id", "name", "category", "type", "is_active"]
-
-    list_select_related = [
-        "category",
-    ]
+    list_display = ["id", "name", "categories", "type", "is_active"]
+    form = ProductAdminForm
 
     inlines = [
         ProductVariantInlineAdmin,
@@ -134,13 +187,29 @@ class ProductAdmin(admin.ModelAdmin):
     list_filter = ["type", "is_active"]
     search_fields = ["name", "category__name"]
 
+    def categories(self, obj):
+        return [category.name for category in obj.category.all()]
+
+    def get_queryset(self, request):
+        return (
+            super()
+            .get_queryset(request)
+            .prefetch_related(
+                "category",
+            )
+        )
+
 
 class ProductAttributeAdmin(admin.ModelAdmin):
+    inlines = [
+        AttributeChoiceInlineAdmin,
+    ]
     list_display = ["id", "name", "type", "is_mandatory"]
     search_fields = [
         "name",
     ]
     list_filter = ["is_mandatory", "type"]
+    exclude = ("attribute_choices",)
 
 
 class AttributeChoiceAdmin(admin.ModelAdmin):
@@ -194,17 +263,19 @@ class AdressAdmin(LeafletGeoAdmin):
     ]
 
 
-class OrderAdmin(admin.ModelAdmin):
+class OrderAdmin(ImportExportModelAdmin, admin.ModelAdmin):
     list_display = [
         "id",
         "customer",
         "status",
+        "type_of_order",
         "payment_method",
         "store",
         "total_amount",
         "pickup_time",
+        "created_at",
     ]
-    inlines = [OrderItemInline]
+    inlines = [OrderItemInline, OrderHistoryInlineAdmin]
     search_fields = [
         "customer__email",
     ]
@@ -216,14 +287,43 @@ class OrderAdmin(admin.ModelAdmin):
         "status",
     ]
 
+    def get_queryset(self, request):
+        queryset = (
+            super()
+            .get_queryset(request)
+            .select_related(
+                "shipping_method",
+                "payment_method",
+                "store",
+                "customer",
+            )
+            .prefetch_related(
+                "items",
+                "items__product_variant__inventories",
+                "items__attribute_choices",
+            )
+        )
+        return queryset
 
-class PaymentAdmin(admin.ModelAdmin):
-    list_display = ("user", "method", "amount", "currency", "status")
+
+class PaymentAdmin(ImportExportModelAdmin, admin.ModelAdmin):
+    list_display = (
+        "id",
+        "user",
+        "method",
+        "amount",
+        "currency",
+        "status",
+        "payment_post_at",
+        "created_at",
+    )
     list_filter = [
         "method__payment_provider",
         "status",
     ]
     search_fields = ["orders__store__name", "user__email"]
+    readonly_fields = ("orders",)
+    date_hierarchy = "created_at"
 
 
 class InventoryAdmin(admin.ModelAdmin):
@@ -232,11 +332,10 @@ class InventoryAdmin(admin.ModelAdmin):
         "variant",
         "store",
         "quantity",
+        "is_snoozed",
         "is_active",
         "price",
         "discount_percent",
-        "is_deliverable",
-        "is_primary",
         "is_uncountable",
         "preparation_time",
     ]
@@ -267,12 +366,15 @@ class TaxAdmin(admin.ModelAdmin):
     ]
 
 
-class WalletTransactionAdmin(admin.ModelAdmin):
+class WalletTransactionAdmin(ImportExportModelAdmin, admin.ModelAdmin):
     list_display = [
         "id",
-        "wallet",
+        "user",
         "type",
         "amount",
+        "is_cashback",
+        "is_refund",
+        "created_at",
     ]
     list_filter = [
         "type",
@@ -281,6 +383,9 @@ class WalletTransactionAdmin(admin.ModelAdmin):
         "wallet__user__email",
     ]
 
+    def user(self, obj) -> typing.Text:
+        return obj.wallet.user.email
+
 
 class WalletAdmin(admin.ModelAdmin):
     list_display = ("id", "user", "balance", "currency")
@@ -293,6 +398,10 @@ class WalletMediaAdmin(admin.ModelAdmin):
     list_display = ("id", "image", "image_thumbnail_medium", "is_active")
 
 
+class AvailabilityHoursAdmin(admin.ModelAdmin):
+    list_display = ("id", "store", "category", "from_hour", "to_hour")
+
+
 admin.site.register(models.Store, StoreAdmin)
 admin.site.register(models.ShippingMethod, ShippingMethodAdmin)
 admin.site.register(models.PaymentMethod, PaymentMethodAdmin)
@@ -311,3 +420,5 @@ admin.site.register(models.Tax, TaxAdmin)
 admin.site.register(models.WalletTransaction, WalletTransactionAdmin)
 admin.site.register(models.Wallet, WalletAdmin)
 admin.site.register(models.WalletMedia, WalletMediaAdmin)
+admin.site.register(models.AvailabilityHours, AvailabilityHoursAdmin)
+admin.site.register(models.Menu, MenuAdmin)
