@@ -10,11 +10,11 @@ logger = logging.getLogger(__name__)
 class TapPaymentManager(models.Manager):
     def create(self, **kwargs):
         # TODO: Add logging to on debug level
-        if (
-            "source" not in kwargs
-            or kwargs.get("source") not in self.model.Sources.values
-        ):
-            raise ValueError(f"Invalid source value. {kwargs.get('source')}")
+        #        if (
+        #            "source" not in kwargs
+        #            or kwargs.get("source") not in self.model.Sources.values
+        #        ):
+        #            raise ValueError(f"Invalid source value. {kwargs.get('source')}")
         source = kwargs.pop("source")
         payment = kwargs.get("payment")
         user = kwargs.get("user")
