@@ -1,6 +1,7 @@
 import json
 import logging
 import typing
+from decimal import Decimal
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
@@ -74,7 +75,12 @@ class PaymentManager(models.Manager):
             gateway = method.payment_provider
         instance: "models.Payment" = super().create(currency=currency, *args, **kwargs)
         instance.orders.set(orders)
-        if gateway in [settings.TAP_CREDIT_CARD, settings.TAP_KNET, settings.TAP_ALL]:
+        if gateway in [
+            settings.TAP_CREDIT_CARD,
+            settings.TAP_KNET,
+            settings.TAP_ALL,
+            settings.APPLE_PAY,
+        ]:
             source = gateway
             TapPayment.objects.create(
                 source=source,
@@ -86,7 +92,7 @@ class PaymentManager(models.Manager):
             try:
                 wallet = kwargs["user"].wallets.get(currency=currency)
             except ObjectDoesNotExist:
-                instance.mark_failed()
+                instance.mark_failed("Wallet Not Found")
                 raise ValidationError({"wallet": _("Wallet Not Found")})
             WalletTransaction.objects.create(
                 wallet=wallet,
@@ -100,7 +106,7 @@ class PaymentManager(models.Manager):
                 gift_card_id=orders[0].extra_infos["gift_card"], payment=instance
             )
             if not response["success"]:
-                instance.mark_failed()
+                instance.mark_failed(response["error"])
                 raise ValidationError({"gift_card": response["error"]})
         instance.mark_paid()
         return instance
@@ -111,13 +117,29 @@ class InventoryManager(ActiveMixin, models.Manager):
 
 
 class OrderItemManager(models.Manager):
-    def create(self, *args, **kwargs):
+    def create(self, attributes=[], *args, **kwargs):
         order_item = super().create(**kwargs)
+        order_item.attribute_choices.set(attributes)
+        inventory_price = (
+            round(float(order_item.inventory.price), 3) if order_item.inventory else 0
+        )
+        attribute_choices_price = sum(
+            map(lambda item: float(item.price) or 0, attributes)
+        )
+        total_price = round(
+            (inventory_price + attribute_choices_price) * int(order_item.quantity), 3
+        )
         init_data = {
-            "inventory_price": round(float(order_item.inventory.price), 3)
-            if order_item.inventory
-            else 0
+            "inventory_price": inventory_price,
+            "product_variant": f"{order_item.product_variant.product.name} {order_item.product_variant.name}",
+            "quantity": order_item.quantity,
+            "attribute_choices": [
+                {"name": attribute.name, "price": round(float(attribute.price), 3)}
+                for attribute in attributes
+            ],
+            "total_price": total_price,
         }
+        order_item.total_price = Decimal(total_price).quantize(Decimal("0.000"))
         order_item.init_data = json.dumps(init_data)
         order_item.save()
         return order_item
