@@ -52,6 +52,21 @@ class Favorite(DjangoModelCleanMixin, models.Model):
             )
         return favorite
 
+    def update_favorite(self, name, extras=[]):
+        from ob_dj_store.core.stores.models._favorite import FavoriteExtra
+
+        self.name = name
+        self.save()
+        FavoriteExtra.objects.filter(favorite=self).delete()
+        for extra in extras:
+            extra_content_type = ContentType.objects.get_for_model(type(extra))
+            self.extras.create(
+                content_type=extra_content_type,
+                object_id=extra.id,
+                content_object=extra,
+            )
+        return self
+
 
 class FavoriteExtra(DjangoModelCleanMixin, models.Model):
     favorite = models.ForeignKey(
