@@ -4,6 +4,7 @@ from decimal import Decimal
 from django.contrib.auth import get_user_model
 from django.core.exceptions import ValidationError
 from django.db import models
+from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
 
 from ob_dj_store.core.stores.managers import (
@@ -42,6 +43,12 @@ class Inventory(DjangoModelCleanMixin, models.Model):
     created_at = models.DateTimeField(auto_now_add=True)
     updated_at = models.DateTimeField(auto_now=True)
     objects = InventoryManager()
+    snooze_start_date = models.DateTimeField(
+        default=now, help_text=_("When snooze status should begin")
+    )
+    snooze_end_date = models.DateTimeField(
+        default=now, help_text=_("When snooze status should end")
+    )
 
     # Add unique together constraint for store and variant
     class Meta:
@@ -52,6 +59,10 @@ class Inventory(DjangoModelCleanMixin, models.Model):
     def __str__(self):
         return f"Inventory - {self.variant.product.name} - {getattr(self.store,'name', None)}"
 
+    @property
+    def is_snoozed(self):
+        return self.snooze_start_date <= now() <= self.snooze_end_date
+
     @property
     def discounted_price(self) -> Decimal:
         if self.discount_percent:
