@@ -95,6 +95,7 @@ class Payment(models.Model):
         decimal_places=settings.DEFAULT_DECIMAL_PLACES,
     )
     currency = models.CharField(_("Currency"), max_length=10)
+    result = models.TextField(null=True)
     created_at = models.DateTimeField(_("Created at"), auto_now_add=True)
     updated_at = models.DateTimeField(_("Updated at"), auto_now=True)
     payment_post_at = models.DateTimeField(_("Payment Post At"), null=True, blank=True)
@@ -141,8 +142,9 @@ class Payment(models.Model):
         self.payment_post_at = timezone.now()
         self.save()
 
-    def mark_failed(self):
+    def mark_failed(self, message):
         self.status = self.PaymentStatus.FAILED
+        self.result = message
         orders = self.orders.all()
         for order in orders:
             order.status = Order.OrderStatus.CANCELLED
