@@ -1,6 +1,7 @@
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.core.validators import RegexValidator
 from django.db import models
+from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
 
 from ob_dj_store.core.stores.managers import (
@@ -8,6 +9,7 @@ from ob_dj_store.core.stores.managers import (
     ProductManager,
     ProductVariantManager,
 )
+from ob_dj_store.core.stores.models._store import Store
 from ob_dj_store.utils.helpers import (
     category_media_upload_to,
     product_media_upload_to,
@@ -63,6 +65,72 @@ class Category(DjangoModelCleanMixin, models.Model):
         return Category.objects.filter(parent=self)
 
 
+class Menu(DjangoModelCleanMixin, models.Model):
+    store = models.OneToOneField(Store, on_delete=models.CASCADE, related_name="menu")
+    categories = models.ManyToManyField(Category, related_name="menus")
+    name = models.CharField(max_length=255, null=True, blank=True)
+
+    # Audit fields
+    created_at = models.DateTimeField(auto_now_add=True)
+    updated_at = models.DateTimeField(auto_now=True)
+
+    def __str__(self) -> str:
+        return f"{self.store.name} Menu(PK={self.pk})"
+
+
+class AvailabilityHours(DjangoModelCleanMixin, models.Model):
+    """
+    AvailabilityHours model linked to Store model and category
+    every store has many opening_hours
+    """
+
+    class Weekdays(models.IntegerChoices):
+        MONDAY = 1, _("Monday")
+        TUESDAY = 2, _("Tuesday")
+        WEDNESDAY = 3, _("Wednesday")
+        THURSDAY = 4, _("Thursday")
+        FRIDAY = 5, _("Friday")
+        SATURDAY = 6, _("Saturday")
+        SUNDAY = 7, _("Sunday")
+
+    weekday = models.IntegerField(choices=Weekdays.choices)
+    from_hour = models.TimeField()
+    to_hour = models.TimeField()
+    store = models.ForeignKey(
+        Store, on_delete=models.CASCADE, related_name="availability_hours"
+    )
+    category = models.ForeignKey(
+        Category, on_delete=models.CASCADE, related_name="availability_hours"
+    )
+    # Audit fields
+    created_at = models.DateTimeField(auto_now_add=True)
+    updated_at = models.DateTimeField(auto_now=True)
+
+    class Meta:
+        ordering = ("weekday", "from_hour")
+        unique_together = (
+            "weekday",
+            "store",
+            "category",
+        )
+
+    def __str__(self) -> str:
+        return f"Availability Hours for store{self.store}"
+
+    def clean(self) -> None:
+        super().clean()
+        if self.from_hour and self.to_hour and (self.from_hour > self.to_hour):
+            hours_difference = (24 - self.from_hour.hour) + self.to_hour.hour
+            if hours_difference > 24:
+                raise ValidationError(
+                    "Invalid availability hours: from_hour should be lower than to_hour"
+                )
+            else:
+                raise ValidationError(
+                    _("From hour should be lower than To hour"), code="invalid"
+                )
+
+
 class ProductTag(models.Model):
     """
     ProductTag is look up table for indexing and filtering
@@ -109,14 +177,15 @@ class Product(DjangoModelCleanMixin, models.Model):
 
     name = models.CharField(max_length=200, help_text=_("Name"), unique=True)
     slug = models.SlugField(max_length=255, unique=True)
+    label = models.CharField(
+        max_length=200, help_text=_("Label"), null=True, blank=True
+    )
     description = models.TextField(null=True, blank=True)
     # TODO: A product can be assigned to multiple categories
-    category = models.ForeignKey(
+    category = models.ManyToManyField(
         Category,
-        on_delete=models.SET_NULL,
         related_name="products",
         blank=True,
-        null=True,
     )
     is_active = models.BooleanField(default=False)
     is_featured = models.BooleanField(default=False)
@@ -127,7 +196,9 @@ class Product(DjangoModelCleanMixin, models.Model):
     )
     plu = models.CharField(max_length=40, unique=True, null=True, blank=True)
     external_id = models.CharField(max_length=40, unique=True, null=True, blank=True)
-
+    order_value = models.PositiveSmallIntegerField(
+        verbose_name=_("ordering"), default=1
+    )
     # Audit fields
     created_at = models.DateTimeField(auto_now_add=True)
     updated_at = models.DateTimeField(auto_now=True)
@@ -145,16 +216,11 @@ class Product(DjangoModelCleanMixin, models.Model):
     class Meta:
         verbose_name = _("Product")
         verbose_name_plural = _("Products")
+        ordering = ("order_value",)
 
     def __str__(self):
         return f"Product {self.name} (PK={self.pk})"
 
-    def save(self, *args, **kwargs):
-        if self.category:
-            if not self.category.parent:
-                raise ValidationError(_("Category Doesn't have a parent"))
-        return super(Product, self).save(*args, **kwargs)
-
 
 # TODO: must remove, redundunt
 class Attribute(DjangoModelCleanMixin, models.Model):
@@ -206,6 +272,33 @@ class AttributeChoice(DjangoModelCleanMixin, models.Model):
         return f"{self.name}: Attribute Choice  (PK={self.pk})"
 
 
+class StoreAttributeChoice(models.Model):
+    store = models.ForeignKey(
+        Store, on_delete=models.CASCADE, related_name="attribute_inventory"
+    )
+    attribute = models.ForeignKey(
+        AttributeChoice, on_delete=models.CASCADE, related_name="attribute_inventory"
+    )
+    snooze_start_date = models.DateTimeField(
+        default=now, help_text=_("When snooze status should begin")
+    )
+    snooze_end_date = models.DateTimeField(
+        default=now, help_text=_("When snooze status should end")
+    )
+
+    class Meta:
+        verbose_name = _("Store Attribute Choice")
+        verbose_name_plural = _("Store Attribute Choices")
+        unique_together = (("store", "attribute"),)
+
+    def __str__(self) -> str:
+        return f"StoreProductAttribute(PK={self.pk})"
+
+    @property
+    def is_snoozed(self):
+        return self.snooze_start_date <= now() <= self.snooze_end_date
+
+
 class ProductAttribute(DjangoModelCleanMixin, models.Model):
     """
     ProductAttribute represent a characteristic -attribute- with is choices -attribute_choices-
@@ -257,6 +350,8 @@ class ProductVariant(DjangoModelCleanMixin, models.Model):
     """
 
     name = models.CharField(max_length=200, help_text=_("Name"))
+    label = models.CharField(max_length=50, help_text=_("Label"), null=True, blank=True)
+    description = models.TextField(null=True, blank=True)
     product = models.ForeignKey(
         Product, related_name="product_variants", on_delete=models.CASCADE
     )
@@ -266,6 +361,7 @@ class ProductVariant(DjangoModelCleanMixin, models.Model):
     sku = models.CharField(max_length=100, null=True, blank=True)
     plu = models.CharField(max_length=40, unique=True, null=True, blank=True)
     external_id = models.CharField(max_length=40, unique=True, null=True, blank=True)
+    is_special = models.BooleanField(default=False)
     image = models.ImageField(
         upload_to=product_variant_media_upload_to,
         null=True,
@@ -286,7 +382,7 @@ class ProductVariant(DjangoModelCleanMixin, models.Model):
         verbose_name_plural = _("Product Variations")
 
     def __str__(self):
-        return f"Product Variation {self.name} (PK={self.pk})"
+        return f"Variation {self.product.name} {self.name} (PK={self.pk})"
 
     @property
     def has_inventory(self):
