@@ -49,6 +49,9 @@ class ShippingMethod(models.Model):
         max_digits=settings.DEFAULT_MAX_DIGITS,
         decimal_places=settings.DEFAULT_DECIMAL_PLACES,
     )
+    order_value = models.PositiveSmallIntegerField(
+        verbose_name=_("ordering"), default=1
+    )
 
     # Audit fields
     created_at = models.DateTimeField(auto_now_add=True)
@@ -59,6 +62,7 @@ class ShippingMethod(models.Model):
     class Meta:
         verbose_name_plural = _("Shipping Methods")
         verbose_name = _("Shipping Method")
+        ordering = ("order_value",)
 
     def __str__(self):
         return f"Shipping Method {self.name} (PK={self.pk})"
@@ -78,6 +82,9 @@ class PaymentMethod(models.Model):
     is_active = models.BooleanField(default=True)
     name = models.CharField(max_length=200, help_text=_("Name"))
     description = models.TextField(null=True, blank=True)
+    order_value = models.PositiveSmallIntegerField(
+        verbose_name=_("ordering"), default=1
+    )
 
     # Audit fields
     created_at = models.DateTimeField(auto_now_add=True)
@@ -88,6 +95,7 @@ class PaymentMethod(models.Model):
     class Meta:
         verbose_name_plural = _("Payment Methods")
         verbose_name = _("Payment Method")
+        ordering = ("order_value",)
 
     def __str__(self):
         return f"Payment Method {self.name} (PK={self.pk})"
@@ -144,6 +152,7 @@ class Store(DjangoModelCleanMixin, models.Model):
             validate_currency,
         ],
     )
+    busy_mode = models.BooleanField(default=False)
 
     created_at = models.DateTimeField(auto_now_add=True)
     updated_at = models.DateTimeField(auto_now=True)
@@ -189,6 +198,7 @@ class OpeningHours(DjangoModelCleanMixin, models.Model):
     # Audit fields
     created_at = models.DateTimeField(auto_now_add=True)
     updated_at = models.DateTimeField(auto_now=True)
+    always_open = models.BooleanField(default=False)
 
     class Meta:
         ordering = ("weekday", "from_hour")
@@ -203,9 +213,15 @@ class OpeningHours(DjangoModelCleanMixin, models.Model):
     def clean(self) -> None:
         super().clean()
         if self.from_hour and self.to_hour and (self.from_hour > self.to_hour):
-            raise ValidationError(
-                _("From hour should be lower than To hour"), code="invalid"
-            )
+            hours_difference = (24 - self.from_hour.hour) + self.to_hour.hour
+            if hours_difference > 24:
+                raise ValidationError(
+                    "Invalid opening hours: from_hour should be lower than to_hour"
+                )
+            else:
+                raise ValidationError(
+                    _("From hour should be lower than To hour"), code="invalid"
+                )
 
 
 class PhoneContact(DjangoModelCleanMixin, models.Model):
